
import java.awt.*;
import java.applet.*;
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

import webwisdom.tango.interfaces.AppletBase;
import webwisdom.tango.messages.*;
import webwisdom.tango.consts.*;

/**
 * An interactive test of the Graphics.drawArc and Graphics.fillArc
 * routines. Can be run either as a standalone application by
 * typing "java ArcTest" or as an applet in the AppletViewer.
 */

//public class ArcTest extends Applet {
public class ArcTest extends AppletBase {

	ArcControls controls;
	boolean	Master = true;

	int	saved_start;
	int	saved_end;
	int	saved_filled;

	public void init() {

		register();
		setLayout(new BorderLayout());
		ArcCanvas c = new ArcCanvas();
		add("Center", c);
		add("South", controls = new ArcControls(c, this));

	}

	public void start() {
		controls.enable();
	}

	public void stop() {
		controls.disable();
	}

	public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY) {
			System.exit(0);
		}
		return false;
	}

	public void receive(AppEventMessage em) {

		StringTokenizer recv_data = null ;
		boolean filled;
		int filled_int;
		int start;
		int end;

		// by default, when a new instance joins, it gets an master/slave message
		if (em.getType() == Const.CONTROL) {
			CAppAnswerEventMessage msg = (CAppAnswerEventMessage) em;
			if (msg.getAnswerType() == CAConsts.ISMASTER) {
				Master = msg.isMaster();
				if (Master) {
					System.out.println("I am master");
				}
				else {
					System.out.println("I am slave");
					String send_data = "Initial";
					byte[] t = new byte[send_data.length()];
					send_data.getBytes(0, send_data.length(), t, 0);
					AppEventMessage emg = new AppEventMessage(Const.DATA, t);
					send(emg);
				}
			}
			return;
		}

		// decode the message into a String
		String r = new String(em.getData(), 0, 0, em.getLen());

		System.out.println("receive string" + r);

		// master sends the inital value of filled, start, end
		if (r.startsWith("Initial") && Master) {
			// make one string data to be sent,
			// convert to byte, and send it
			String send_data = saved_filled + 
				" " + saved_start + " " + saved_end;

			byte[] t = new byte[send_data.length()];
			send_data.getBytes(0, send_data.length(), t, 0);
			AppEventMessage emg = new AppEventMessage(Const.DATA, t);
			send(emg);
			return;
		}

		// parse the received String
		recv_data = new StringTokenizer(r);

		filled_int = (Integer.valueOf(recv_data.nextToken())).intValue();
		start      = (Integer.valueOf(recv_data.nextToken())).intValue();
		end        = (Integer.valueOf(recv_data.nextToken())).intValue();

		if ( filled_int == 1 ) {
			filled = true;
		}
		else {
			filled = false;
		}

		controls.draw_canvas(filled, start, end);
		controls.s.setText(String.valueOf(start));
		controls.e.setText(String.valueOf(end));

	}
}

class ArcCanvas extends Canvas {
	int		startAngle = 0;
	int		endAngle = 45;
	boolean	filled = false;
	Font	font;

	public void paint(Graphics g) {
		Rectangle r = bounds();
		int hlines = r.height / 10;
		int vlines = r.width / 10;

		g.setColor(Color.pink);
		for (int i = 1; i <= hlines; i++) {
			g.drawLine(0, i * 10, r.width, i * 10);
		}
		for (int i = 1; i <= vlines; i++) {
			g.drawLine(i * 10, 0, i * 10, r.height);
		}

		g.setColor(Color.red);
		if (filled) {
			g.fillArc(0, 0, r.width - 1, r.height - 1, startAngle, endAngle);
		}
		else {
			g.drawArc(0, 0, r.width - 1, r.height - 1, startAngle, endAngle);
		}

		g.setColor(Color.black);
		g.setFont(font);
		g.drawLine(0, r.height / 2, r.width, r.height / 2);
		g.drawLine(r.width / 2, 0, r.width / 2, r.height);
		g.drawLine(0, 0, r.width, r.height);
		g.drawLine(r.width, 0, 0, r.height);
		int sx = 10;
		int sy = r.height - 28;
		g.drawString("S = " + startAngle, sx, sy); 
		g.drawString("E = " + endAngle, sx, sy + 14); 
	}

	public void redraw(boolean filled, int start, int end) {
		this.filled = filled;
		this.startAngle = start;
		this.endAngle = end;
		repaint();
	}
}

class ArcControls extends Panel {
	TextField s;
	TextField e;
	ArcCanvas canvas;
	ArcTest test;

	public ArcControls(ArcCanvas canvas, ArcTest test) {
		this.canvas = canvas;
		this.test = test;
		add(s = new TextField("0", 4));
		add(e = new TextField("45", 4));
		add(new Button("Fill"));
		add(new Button("Draw"));
	}

	public boolean action(Event ev, Object arg) {

		String send_data = null;
		int button_action;
		int int_start;
		int int_end;
		boolean filled;

/**
		// only master can act.
		if (!test.Master) {
			return true;
		}
**/

		if (ev.target instanceof Button) {
			String label = (String)arg;

			filled = label.equals("Fill");
			if (filled) {
				button_action = 1;
			}
			else {
				button_action = 0;
			}

			int_start = Integer.parseInt(s.getText().trim());
			int_end   = Integer.parseInt(e.getText().trim());

			test.saved_start	= int_start;
			test.saved_end		= int_end;
			test.saved_filled	= button_action;

			// make one string data to be sent,
			// convert to byte, and send it
			send_data = button_action + " " + int_start + " " + int_end;

			byte[] t = new byte[send_data.length()];
			send_data.getBytes(0, send_data.length(), t, 0);
			AppEventMessage em = new AppEventMessage(Const.DATA, t);
			test.send(em);

			canvas.redraw(filled, int_start, int_end);

			return true;
		}

		return false;
	}

	public void draw_canvas(boolean filled, int start, int end) {
		canvas.redraw(filled, start, end);
	}
}
	
